#include <msp/gl/matrix.h>
#include <msp/gl/meshbuilder.h>
#include <msp/gl/renderer.h>
#include "selection.h"
#include "selectionwrap.h"

using namespace std;
using namespace Msp;
using namespace R2C2;

SelectionWrap::SelectionWrap(Selection &s):
	selection(s)
{
	selection.signal_changed.connect(sigc::mem_fun(this, &SelectionWrap::selection_changed));
}

SelectionWrap::~SelectionWrap()
{
	for(map<const ObjectType *, GL::Mesh *>::iterator i=meshes.begin(); i!=meshes.end(); ++i)
		delete i->second;
	for(map<const Object *, GL::Mesh *>::iterator i=transient_meshes.begin(); i!=transient_meshes.end(); ++i)
		delete i->second;
}

void SelectionWrap::render(GL::Renderer &renderer, const GL::Tag &) const
{
	for(list<Wrap>::const_iterator i=wraps.begin(); i!=wraps.end(); ++i)
	{
		GL::MatrixStack::Push push(renderer.matrix_stack());
		const Vector &pos = i->object->get_position();
		renderer.matrix_stack() *= GL::Matrix::translation(pos+GL::Vector3(0, 0, 0.001));
		renderer.matrix_stack() *= GL::Matrix::rotation(i->object->get_rotation(), 0, 0, 1);
		i->mesh->draw(renderer);
	}
}

void SelectionWrap::selection_changed()
{
	wraps.clear();
	const set<Object *> &objects = selection.get_objects();
	for(set<Object *>::iterator i=objects.begin(); i!=objects.end(); ++i)
	{
		Wrap wrap;
		wrap.object = *i;
		wrap.mesh = &get_mesh(**i);
		wraps.push_back(wrap);
	}

	set<const Object *> cobjects(objects.begin(), objects.end());
	for(map<const Object *, GL::Mesh *>::iterator i=transient_meshes.begin(); i!=transient_meshes.end(); )
	{
		if(!cobjects.count(i->first))
		{
			delete i->second;
			transient_meshes.erase(i++);
		}
		else
			++i;
	}
}

const GL::Mesh &SelectionWrap::get_mesh(const Object &obj)
{
	map<const Object *, GL::Mesh *>::iterator i = transient_meshes.find(&obj);
	if(i!=transient_meshes.end())
		return *i->second;

	const ObjectType &type = obj.get_type();
	map<const ObjectType *, GL::Mesh *>::iterator j = meshes.find(&type);
	if(j!=meshes.end())
		return *j->second;

	GL::Mesh *mesh = new GL::Mesh((GL::COLOR4_UBYTE, GL::VERTEX2));
	GL::MeshBuilder bld(*mesh);
	bld.color(0.0f, 1.0f, 0.0f, 1.0f);

	const Shape *shape = obj.get_shape();
	if(!shape)
		throw logic_error("No shape");
	
	Geometry::BoundingBox<float, 3> bbox = shape->get_axis_aligned_bounding_box();
	const Vector &min_pt = bbox.get_minimum_point();
	const Vector &max_pt = bbox.get_maximum_point();

	bld.begin(GL::LINE_LOOP);
	bld.vertex(min_pt.x-0.005, min_pt.y-0.005);
	bld.vertex(max_pt.x+0.005, min_pt.y-0.005);
	bld.vertex(max_pt.x+0.005, max_pt.y+0.005);
	bld.vertex(min_pt.x-0.005, max_pt.y+0.005);
	bld.end();

	if(shape!=type.get_shape())
		transient_meshes[&obj] = mesh;
	else
		meshes[&type] = mesh;

	return *mesh;
}
