#include <msp/core/maputils.h>
#include <msp/gl/tests.h>
#include <msp/gltk/button.h>
#include <msp/gltk/image.h>
#include <msp/gltk/label.h>
#include <msp/strings/format.h>
#include "libr2c2/vehicle.h"
#include "libr2c2/vehicletype.h"
#include "engineer.h"
#include "trainview.h"

using namespace Msp;
using namespace R2C2;

TrainView::TrainView(Engineer &e, const Train &t):
	DynamicDialog(e),
	train(t),
	mode(SIDE),
	forward(true),
	view(engineer.get_layout_3d(), 400, 300)
{
	Loader::WidgetMap widgets;
	DataFile::load(*this, "data/trainview.ui", widgets);

	dynamic_cast<GLtk::Label *>(get_item(widgets, "lbl_title"))->set_text(format("View of %s", train.get_name()));
	dynamic_cast<GLtk::Toggle *>(get_item(widgets, "tgl_forward"))->signal_toggled.connect(sigc::mem_fun(this, &TrainView::set_forward));
	dynamic_cast<GLtk::Toggle *>(get_item(widgets, "tgl_roof"))->signal_toggled.connect(sigc::bind(sigc::mem_fun(this, &TrainView::ui_mode_toggled), ROOF));
	dynamic_cast<GLtk::Toggle *>(get_item(widgets, "tgl_side"))->signal_toggled.connect(sigc::bind(sigc::mem_fun(this, &TrainView::ui_mode_toggled), SIDE));
	dynamic_cast<GLtk::Toggle *>(get_item(widgets, "tgl_head"))->signal_toggled.connect(sigc::bind(sigc::mem_fun(this, &TrainView::ui_mode_toggled), HEAD));

	tex.set_min_filter(GL::LINEAR);
	tex.storage(GL::RGB, 400, 300);
	tex.image(0, GL::RGB, GL::UNSIGNED_BYTE, 0);
	fbo.attach(GL::COLOR_ATTACHMENT0, tex, 0);
	depth.storage(GL::DEPTH_COMPONENT, 400, 300);
	fbo.attach(GL::DEPTH_ATTACHMENT, depth);

	view.get_camera().set_depth_clip(0.01, 10);

	dynamic_cast<GLtk::Image *>(get_item(widgets, "img_view"))->set_image(&tex);
}

void TrainView::set_mode(Mode m)
{
	mode = m;
}

void TrainView::ui_mode_toggled(bool v, Mode m)
{
	if(v)
		set_mode(m);
}

void TrainView::set_forward(bool f)
{
	forward = f;
}

void TrainView::update()
{
	const Vehicle &veh = train.get_vehicle(0);
	const Vector &pos = veh.get_position();
	Angle angle = veh.get_rotation();
	if(!forward)
		angle += Angle::half_turn();
	Vector fwd_vec = rotated_vector(Vector(1, 0, 0), angle);
	Vector side_vec = rotated_vector(Vector(0, -1, 0), angle);
	float l = veh.get_type().get_length();

	GL::Camera &camera = view.get_camera();
	switch(mode)
	{
	case ROOF:
		camera.set_position(pos-l*fwd_vec+Vector(0, 0, 0.07));
		camera.set_look_direction(fwd_vec-Vector(0, 0, 0.2));
		break;
	case SIDE:
		camera.set_position(pos-0.08f*fwd_vec+0.05f*side_vec+Vector(0, 0, 0.03));
		camera.set_look_direction(fwd_vec-side_vec*0.2f);
		break;
	case HEAD:
		camera.set_position(pos+fwd_vec*(l*0.55f)+Vector(0, 0, 0.03));
		camera.set_look_direction(fwd_vec);
		break;
	}

	GL::Bind _bind_fbo(fbo);
	fbo.clear(GL::COLOR_BUFFER_BIT|GL::DEPTH_BUFFER_BIT);
	view.render();
}
