#ifndef LIBR2C2_TRACKAPPEARANCE_H_
#define LIBR2C2_TRACKAPPEARANCE_H_

#include <string>
#include <msp/datafile/objectloader.h>
#include "profile.h"

namespace R2C2 {

class TrackAppearance
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<TrackAppearance>
	{
	public:
		Loader(TrackAppearance &);

	private:
		void ballast_profile();
		void gauge(float);
		void rail_profile();
		void tie_length(float);
		void tie_profile();
		void tie_spacing(float);
	};

private:
	float gauge;
	Profile rail_profile;
	Profile ballast_profile;
	Profile tie_profile;
	float tie_length;
	float tie_spacing;
	std::string technique;

public:
	TrackAppearance();

	float get_gauge() const { return gauge; }
	float get_rail_elevation() const;
	const Profile &get_rail_profile() const { return rail_profile; }
	const Profile &get_ballast_profile() const { return ballast_profile; }
	const Profile &get_tie_profile() const { return tie_profile; }
	float get_tie_length() const { return tie_length; }
	float get_tie_spacing() const { return tie_spacing; }
	const std::string &get_technique() const { return technique; }
};

} // namespace R2C2

#endif
