#ifndef LIBR2C2_TRACKITER_H_
#define LIBR2C2_TRACKITER_H_

#include <set>
#include <msp/core/refptr.h>
#include "tracktype.h"

namespace R2C2 {

class BlockIter;
class Track;

/**
An iterator for traversing tracks.
*/
class TrackIter
{
private:
	Track *_track;
	unsigned _entry;

public:
	TrackIter();
	TrackIter(Track *, unsigned);

	Track *track() const { return _track; }
	unsigned entry() const { return _entry; }
	BlockIter block_iter() const;
	const TrackType::Endpoint &endpoint() const;

private:
	int get_exit(unsigned) const;
public:
	TrackIter next() const;
	TrackIter next(unsigned) const;
	TrackIter reverse() const;
	TrackIter reverse(unsigned) const;
	TrackIter flip() const;

	Track &operator*() const;
	Track *operator->() const { return _track; }
	bool operator==(const TrackIter &) const;
	bool operator!=(const TrackIter &other) const { return !(*this==other); }
	operator bool() const { return _track!=0; }
};


/**
A track iterator that detects looping.

A list of visited tracks is maintained internally to the iterator.  This list
is shared between iterators as long as next() is only called once per iterator.
Subsequent calls to next() cause the head of the list to be copied.
*/
class TrackLoopIter: public TrackIter
{
private:
	typedef std::list<Track *> TrackList;

	Msp::RefPtr<TrackList> _visited;
	TrackList::iterator _last;
	bool _looped;

public:
	TrackLoopIter();
	TrackLoopIter(Track *, unsigned);
	TrackLoopIter(const TrackIter &);
private:
	TrackLoopIter(const TrackIter &, Msp::RefPtr<TrackList>, const TrackList::iterator &);

public:
	bool looped() const { return _looped; }

	TrackLoopIter next() const;
	TrackLoopIter next(unsigned) const;
};

} // namespace R2C2

#endif
