#include <map>
#include <msp/core/application.h>
#include <msp/core/getopt.h>
#include <msp/datafile/loader.h>
#include <msp/datafile/parser.h>
#include <msp/io/print.h>

using namespace std;
using namespace Msp;

class ShoppingList: public RegisteredApplication<ShoppingList>
{
private:
	class InventoryLoader: public DataFile::Loader
	{
	public:
		InventoryLoader(ShoppingList &);
	private:
		ShoppingList &sl;

		void track(const string &, int);
	};

	class LayoutLoader: public DataFile::Loader
	{
	public:
		LayoutLoader(ShoppingList &);
	private:
		ShoppingList &sl;

		void track(const string &);
	};

	map<string, unsigned> inventory;
	map<string, unsigned> layout;

public:
	ShoppingList(int, char **);
	int main();
private:
	void load_inventory(const string &);
	void load_layout(const string &);
};


ShoppingList::ShoppingList(int argc, char **argv)
{
	string inv_fn = "inventory";
	GetOpt getopt;
	getopt.add_option('i', "inventory", inv_fn, GetOpt::REQUIRED_ARG);
	getopt(argc, argv);

	load_inventory(inv_fn);
	load_layout(getopt.get_args().front());
}

int ShoppingList::main()
{
	IO::print("// Need to get:\n");
	for(map<string, unsigned>::iterator i=layout.begin(); i!=layout.end(); ++i)
	{
		map<string, unsigned>::iterator j=inventory.find(i->first);
		if(j!=inventory.end())
		{
			if(j->second<i->second)
				IO::print("track \"%s\" %d;\n", i->first, i->second-j->second);
		}
		else
			IO::print("track \"%s\" %d;\n", i->first, i->second);
	}

	IO::print("// Pre-existing:\n");
	for(map<string, unsigned>::iterator i=layout.begin(); i!=layout.end(); ++i)
	{
		map<string, unsigned>::iterator j=inventory.find(i->first);
		if(j!=inventory.end())
			IO::print("track \"%s\" %d;\n", i->first, min(i->second, j->second));
	}

	IO::print("// Unused:\n");
	for(map<string, unsigned>::iterator i=inventory.begin(); i!=inventory.end(); ++i)
	{
		map<string, unsigned>::iterator j=layout.find(i->first);
		if(j!=layout.end())
		{
			if(j->second<i->second)
				IO::print("track \"%s\" %d;\n", i->first, i->second-j->second);
		}
		else
			IO::print("track \"%s\" %d;\n", i->first, i->second);
	}

	return 0;
}

void ShoppingList::load_inventory(const string &fn)
{
	IO::File in(fn);
	DataFile::Parser parser(in, fn);
	InventoryLoader il(*this);
	il.load(parser);
}

void ShoppingList::load_layout(const string &fn)
{
	IO::File in(fn);
	DataFile::Parser parser(in, fn);
	LayoutLoader ll(*this);
	ll.load(parser);
}


ShoppingList::InventoryLoader::InventoryLoader(ShoppingList &s):
	sl(s)
{
	add("track", &InventoryLoader::track);
}

void ShoppingList::InventoryLoader::track(const string &n, int count)
{
	sl.inventory[n] += count;
}


ShoppingList::LayoutLoader::LayoutLoader(ShoppingList &s):
	sl(s)
{
	add("track", &LayoutLoader::track);
	add("base");
	add("beamgate");
	add("route");
	add("signal");
	add("terrain");
	add("zone");
}

void ShoppingList::LayoutLoader::track(const string &n)
{
	++sl.layout[n];
}
